%  xdcr2.m                   T.L.Szabo    Version 2 January 2015          
% this program uses the KLM model to simulate transducer impedance
% and acoustical, electrical and transdcuer loss for
% Chapter 5 in Diagnostic Ultrasound Imaging: Inside Out, second edition
% Version 2 KLM transducer equivalent circuit transducer 
%  this version replaces xdcr.m
% Needs function tloss.m to run (note in rest of program "tloss" is
% used differently to mean transducer loss)
%  Note the area should be A=7 e-5 m^2 correcting an error on p. 143
%  Note the caption for fig 5.14 should have the same area A=7 e-5 m^2 
%  and fig 5.14b depicts the aloss,eloss and tloss curves with tuning in dB

%reset and initialize everything
clear all;
close all;

%  INPUT VARIABLES
% figures set f parameter = 1 to see figure or =0 for no figure


% for displaying a figure, set the corresponding f parameter below to 1
% to see figure & set f parameter to =0 for no figure
fzinr=0; % plot real zinr(+) and imag zinr(*) [Rayls-m^2]vs f(Hz)
fzinl=0; % plot real zinl(+) and imag zinl(*) [Rayls-m^2]vs f(Hz)
fjunct=0; % plot real zin56(+) and imag zin56(*)[Rayls-m^2] vs f(Hz)
fza=0;  % plot real zin45(+) and imag zin45(*)[ohms]vs f(Hz)
fzt=1;  % plot Ra=real (ZT)(+) and Xa+X0=imag (ZT)(*)[ohms] vs f(Hz)
fztune=1.;% plot tuned real (ZT+Xs)(+) and tuned imag (ZT+Xs)(*)[ohms] vs f(Hz)
feloss=0; % plot eloss(*) & eloss2 (+) in dB vs f (Hz)
floss=1;  % plot aloss(+) and eloss(*) and tloss(x) in dB vs f(Hz)


% input variables
eps0=8.85e-12;
pi2=2*pi;
pid2=pi/2;

%  loading conditions
zg=50.;                 % source electrical impedance (ohms)
zf=50.;                 % receive electrical impedance (ohms)z0
zbi=6.;                 % backing impedance (MegaRayls-m^2)
zwi=1.5;                % body or water impedance (MegaRayls-m^2)

%transducer parameters
eli=70000.;            % element length (um)  new 
ewi=1000.;               % element width (um)   new
edi=662.;               % element thickness (um)
akt=.698;               % piezoelectric coupling constant
v0i=3.97;               % crystal velocity (mm/us)
z0i=29.78;              % crystal impedance (MegaRayls-m^2)
epsr=1475;              % clamped relative dielectric constant
a1=0.0;                 % absorption of crystal (nepers/m-Hz)

% matching layer parameters
ml=0;                   %set to=1 for matching layer, =0 for none
%ml=input('enter the number of matching layers(right side):');
aml1=0.0;               % matching layer 1 absorption
dml1i=250.;             % matching layer 1 thickness (um) 
vml1i=3.0;              % matching layer 1 velocity (mm/us)
zml1i=6.68;             % matching layer 1 impedance (MegaRayls-m^2)


%Tuning parameters
ls=0.0;                % no tuning series inductor (Henries)
%ls=2.04*1e-6;                 % tuning series inductor (Henries)
rs=0.0;                 % series inductor resistance

%Area normalization and units scale correction
v0=v0i*1e3;
ew=ewi*1e-6;
el=eli*1e-6;
ed=edi*1e-6;
ed2=ed/2;
area=ew*el;
akt2=akt*akt;
z0=z0i*area*1e6;
zb=zbi*area*1e6;
zw=zwi*area*1e6;
vml1=vml1i*1e3;
dml1=dml1i*1e-6;
zml1=zml1i*area*1e6;

%frequency parameters
nmax=511;
nup=49;
f0=v0/(2*ed);           % resonance frequency (Hz)
f1=.1e6;
dltaf=.1e6;
dltafn=dltaf/f0;
f1n=f1/f0;
w0=pi2*f0;
f=f1*[1 :nmax];         % sets frequency range
fsize=ones(size(f));    % size of frequency array
w=pi2*f;                % omega angular frequency
fn=f/f0;                % normalized frequency parameter
fend=50;                %sets range for freq. plots:100 is 10 MHz,50 is 5MHz,etc

% Capacitance
c0=epsr*eps0*area/ed;   % clamped capacitance
x0=1/(w0*c0);           % clamped cap. reactance

%  tissue or water side (right)
% half thickness crystal layer propagation factor gamma
gamd2=complex(-a1*f*ed2,pid2*f/f0);

% right side

if ml==0
[vr,zinr] = tloss(gamd2,v0,ed2,z0,zw);    % call tl function
   
elseif ml==1
    gamr1=complex(-aml1*f*dml1,pi2*f*dml1/vml1);
    [vr1,zinr1] = tloss(gamr1,vml1,dml1,zml1,zw);    % call tl function
    [vr,zinr] = tloss(gamd2,v0,ed2,z0,zinr1);    % call tl function
else
   noise=input('error! only values of ml=0 or 1 allowed')
end

%  left side
[vl,zinl] = tloss(gamd2,v0,ed2,z0,zb);    % call tl function

% Acoustic Loss (acoustic power to the right div. by total ac. power
% calc. ratio of absolute right to left impedances squared as intermediate param.

rzrzl=(abs(zinr).*abs(zinr))./(abs(zinl).*abs(zinl)); 
aloss=1*fsize./(1*fsize+(real(zinl).*rzrzl./real(zinr)));

if fzinr==1
   figure;
   plot(f(1:fend),real(zinr(1:fend)),'b+-',f(1:fend),imag(zinr(1:fend)),'r*--');
   title('real zinr(+) and imag zinr(*) [Rayls-m^2]vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zinr)','imag(zinr)',0);
end

if fzinl==1
   figure;
   plot(f(1:fend),real(zinl(1:fend)),'b+-',f(1:fend),imag(zinl(1:fend)),'g*--');
   title('real zinl(+) and imag zinl(*)[Rayls-m^2] vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zinl)','imag(zinl)',0);
end

% acoustic junction matrix56
a56=1.0;
b56=0.0;
c56=1./zinl;
d56=1.0;
zin56=(a56*zinr+b56*fsize)./(c56.*zinr+d56*fsize);
v56=zinr./(a56*zinr+b56*fsize);

if fjunct==1
   figure;
   plot(f(1:fend),real(zin56(1:fend)),'b+-',f(1:fend),imag(zin56(1:fend)),'g*--');
   title('real zin56(+) and imag zin56(*)[Rayls-m^2] vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zin56)','imag(zin56)',0);
end

% electro-acoustic transformer matrix 45
phi=akt*sqrt(pi/(w0*c0*z0))*sin(pid2*fn+eps)./(pid2*fn+eps);
a45=phi;
b45=0.0;
c45=0.0;
d45=1./phi;
zin45=(a45.*zin56+b45*fsize)./(c45*zin56+d45);
v45=zin56./(a45.*zin56+b45*fsize);

if fza==1
   figure;
   plot(f(1:fend),real(zin45(1:fend)),'b+-',f(1:fend),imag(zin45(1:fend)),'g*--');
   title('real zin45(+) and imag zin45(*)[ohms]vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zin45)','imag(zin45)',0);
end

% double capacitor matrix 34
cp=-c0*fsize./(akt2*sin(pi*fn+eps*fsize)./(pi*fn+eps*fsize));
c0cp=complex(0.*fsize,w*c0.*cp);
a34=1.0;
b34=(c0+cp)./c0cp;
c34=0.0;
d34=1.0;
zin34=(a34*zin45+b34)./(c34*zin45+d34*fsize);  % transducer impedance
zt=zin34;                                      %  transducer impedance
v34=zin45./(a34*zin45+b34);

if fzt==1
   figure;
   plot(f(1:fend),real(zin34(1:fend)),'b+-',f(1:fend),imag(zin34(1:fend)),'g*--');
   title('Ra=real (ZT)(+) and Xa+X0=imag (ZT)(*)[ohms] vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zt)','imag(zt)',0);
end

% electrical input
xs=pi2*f*ls;

%  series tuning
a23=1;
b23=complex(rs,xs)+zg;
c23=0.0;
d23=1.0;
zin23=(a23*zin34+b23)./(c23*zin34+d23*fsize);
zts=zin23-zg*fsize;         % tuned transducer impedance
v23=zin34./(a23*zin34+b23);

if fztune==1
   figure;
   plot(f(1:fend),real(zts(1:fend)),'b+-',f(1:fend),imag(zts(1:fend)),'g*--');
   title('tuned real (ZT+Xs)(+) and tuned imag (ZT+Xs)(*)[ohms] vs f(Hz)');
   xlabel('Frequency (Hz)');ylabel('Impedance (ohms)');
   legend('real(zts)','imag(zts)',0);
end

% electrical loss
eloss=abs(v23).*abs(v23)*4.*real(zt)*zg./(abs(zt).*abs(zt));
rdenom=real(zt)+zg*fsize+rs*fsize;
idenom=imag(zt)+xs.*fsize;
eloss2=4*zg*real(zt)./((rdenom).*(rdenom)+(idenom.*idenom));

if feloss==1
   figure;
   plot(f(1:fend),eloss(1:fend),'m*--',f(1:fend),eloss2(1:fend),'c+-');
   title('eloss(*) & eloss2 (+) in dB vs f (Hz)');
   xlabel('Frequency (Hz)');ylabel('Loss (dB)');
   legend('eloss','eloss2',0);
end

alossdb=10*log10(aloss);
elossdb=10*log10(eloss);
tlossdb=alossdb+elossdb;

if floss==1
   figure;
plot(f(1:fend),alossdb(1:fend),'c+-',f(1:fend),elossdb(1:fend),'m*--',f(1:fend),tlossdb(1:fend),'gx:');
title('Losses in dB vs f(Hz)');
%title('aloss(+) and eloss(*) and tloss(x) in dB vs f(Hz)');xlabel('Frequency (Hz)');ylabel('Loss (dB)');
legend('aloss','eloss','tloss',0);
xlabel('Frequency (Hz)');ylabel('Loss (dB)');
end

